/*******************************************************************************

        @file GrowBuffer.d

        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      
        @version        Initial version; March 2004

        @author         Kris


*******************************************************************************/

module mango.io.GrowBuffer;

private import  mango.io.Buffer;

/*******************************************************************************

        Subclass to provide support for content growth. This is handy when
        you want to keep a buffer around as a scratchpad.

*******************************************************************************/

class GrowBuffer : Buffer
{
        /***********************************************************************
        
                Create a GrowBuffer with the specified initial size.

        ***********************************************************************/

        this (uint size)
        {
                super (size);
        }

        /***********************************************************************

                Overridable method to grow the buffer to the specified size
                when it becomes full. GrowableBuffer.grow(size) will not grow
                the buffer by a factor larger than 2 for each call.
        
        ***********************************************************************/

        protected override bool grow (uint size)
        {
                if (data.length < uint.max / 2)
                   {
                   uint i = data.length * 2;
                   if (i < size)
                       i = size;
                   data.length = i;
                   capacity = i;
                   return true;
                   }
                return false;
        }
}
